@@ -30,6 +30,7 @@
 
 #include "file-data.h"
 #include "file-utils.h"
+#include "glib-utils.h"
 #include "fr-command.h"
 #include "fr-command-lha.h"
 
@@ -91,22 +92,13 @@ mktime_from_string (char *month,
 }
 
 
-static char *
-eat_spaces (char *line)
-{
-	while ((*line == ' ') && (*line != 0))
-		line++;
-	return line;
-}
-
-
 static char **
 split_line_lha (char *line)
 {
-	char **fields;
-	int    n_fields = 7;
-	char  *scan, *field_end;
-	int    i;
+	char       **fields;
+	int          n_fields = 7;
+	const char  *scan, *field_end;
+	int          i;
 
 	fields = g_new0 (char *, n_fields + 1);
 	fields[n_fields] = NULL;
@@ -117,29 +109,39 @@ split_line_lha (char *line)
 		fields[i++] = g_strdup ("");
 		fields[i++] = g_strdup ("");
 		line += strlen ("[MS-DOS]");
+
+	} else if (strncmp (line, "[generic]", 9) == 0) {
+		fields[i++] = g_strdup ("");
+		fields[i++] = g_strdup ("");
+		line += strlen ("[generic]");
 	}
 
 	scan = eat_spaces (line);
 	for (; i < n_fields; i++) {
 		field_end = strchr (scan, ' ');
-		fields[i] = g_strndup (scan, field_end - scan);
-		scan = eat_spaces (field_end);
+		if (field_end != NULL) {
+			fields[i] = g_strndup (scan, field_end - scan);
+			scan = eat_spaces (field_end);
+		}
 	}
 
 	return fields;
 }
 
 
-static char *
-get_last_field (char *line)
+static const char *
+get_last_field_lha (char *line)
 {
-	int   i;
-	char *field;
-	int   n = 7;
+	int         i;
+	const char *field;
+	int         n = 7;
 
 	if (strncmp (line, "[MS-DOS]", 8) == 0) 
 		n--;
 
+	if (strncmp (line, "[generic]", 9) == 0) 
+		n--;
+
 	field = eat_spaces (line);
 	for (i = 0; i < n; i++) {
 		field = strchr (field, ' ');
@@ -154,17 +156,17 @@ static void
 process_line (char     *line, 
 	      gpointer  data)
 {
-	FileData   *fdata;
-	FRCommand  *comm = FR_COMMAND (data);
-	char      **fields;
-	char       *name_field;
+	FileData    *fdata;
+	FRCommand   *comm = FR_COMMAND (data);
+	char       **fields;
+	const char  *name_field;
 
 	g_return_if_fail (line != NULL);
 
 	fdata = file_data_new ();
 
 	fields = split_line_lha (line);
-	fdata->size = atol (fields[2]);
+	fdata->size = g_ascii_strtoull (fields[2], NULL, 10);
 	fdata->modified = mktime_from_string (fields[4], 
 					      fields[5], 
 					      fields[6]);
@@ -172,7 +174,7 @@ process_line (char     *line,
 
 	/* Full path */
 
-	name_field = get_last_field (line);
+	name_field = get_last_field_lha (line);
 
 	if (*name_field == '/') {
 		fdata->full_path = g_strdup (name_field);
@@ -186,7 +188,6 @@ process_line (char     *line,
 
 	fdata->name = g_strdup (file_name_from_path (fdata->full_path));
 	fdata->path = remove_level_from_path (fdata->full_path);
-	fdata->type = gnome_vfs_mime_type_from_name_or_default (fdata->name, GNOME_VFS_MIME_TYPE_UNKNOWN);
 
 	if (*fdata->name == 0)
 		file_data_free (fdata);
@@ -196,13 +197,13 @@ process_line (char     *line,
 
 
 static void
-fr_command_lha_list (FRCommand *comm)
+fr_command_lha_list (FRCommand  *comm,
+		     const char *password)
 {
 	fr_process_set_out_line_func (FR_COMMAND (comm)->process, 
 				      process_line,
 				      comm);
 
-	fr_process_clear (comm->process);
 	fr_process_begin_command (comm->process, "lha");
 	fr_process_add_arg (comm->process, "lq");
 	fr_process_add_arg (comm->process, comm->e_filename);
@@ -311,6 +312,8 @@ fr_command_lha_class_init (FRCommandLhaClass *class)
 static void 
 fr_command_lha_init (FRCommand *comm)
 {
+	comm->file_type = FR_FILE_TYPE_LHA;
+
 	comm->propAddCanUpdate             = TRUE; 
 	comm->propAddCanReplace            = TRUE; 
 	comm->propExtractCanAvoidOverwrite = FALSE;
@@ -336,7 +339,7 @@ fr_command_lha_finalize (GObject *object)
 GType
 fr_command_lha_get_type ()
 {
-        static guint type = 0;
+        static GType type = 0;
 
         if (! type) {
                 GTypeInfo type_info = {
@@ -367,6 +370,10 @@ fr_command_lha_new (FRProcess  *process,
 {
 	FRCommand *comm;
 
+	if (!is_program_in_path("lha")) {
+		return NULL;
+	}
+
 	comm = FR_COMMAND (g_object_new (FR_TYPE_COMMAND_LHA, NULL));
 	fr_command_construct (comm, process, filename);
 
