@@ -39,9 +39,10 @@
 #include "window.h"
 #include "typedefs.h"
 #include "preferences.h"
+#include "file-data.h"
 #include "main.h"
 
-static void     prepare_app         (poptContext pctx);
+static void     prepare_app         (void);
 static void     initialize_data     (void);
 static void     release_data        (void);
 
@@ -55,6 +56,8 @@ GList        *command_list = NULL;
 gint          force_directory_creation;
 GHashTable   *programs_cache = NULL;
 
+static gchar **remaining_args;
+
 static gchar *add_to = NULL;
 static gint   add;
 static gchar *extract_to = NULL;
@@ -130,36 +133,40 @@ FRCommandDescription tar_command_desc[] = {
 };
 
 
-struct poptOption options[] = {
-	{ "add-to", 'a', POPT_ARG_STRING, &add_to, 0,
+static const GOptionEntry options[] = {
+	{ "add-to", 'a', 0, G_OPTION_ARG_STRING, &add_to,
 	  N_("Add files to the specified archive and quit the program"),
 	  N_("ARCHIVE") },
 
-	{ "add", 'd', POPT_ARG_NONE, &add, 0,
+	{ "add", 'd', 0, G_OPTION_ARG_NONE, &add,
 	  N_("Add files asking the name of the archive and quit the program"),
-	  0 },
+	  NULL },
 
-	{ "extract-to", 'e', POPT_ARG_STRING, &extract_to, 0,
-          N_("Extract archives to the specified folder and quit the program"),
-          N_("FOLDER") },
+	{ "extract-to", 'e', 0, G_OPTION_ARG_STRING, &extract_to,
+	  N_("Extract archives to the specified folder and quit the program"),
+	  N_("FOLDER") },
 
-	{ "extract", 'f', POPT_ARG_NONE, &extract, 0,
-          N_("Extract archives asking the destination folder and quit the program"),
-          0 },
+	{ "extract", 'f', 0, G_OPTION_ARG_NONE, &extract,
+	  N_("Extract archives asking the destination folder and quit the program"),
+	  NULL },
 
-	{ "extract-here", 'h', POPT_ARG_NONE, &extract_here, 0,
-          N_("Extract archives using the archive name as destination folder and quit the program"),
-          0 },
+	{ "extract-here", 'h', 0, G_OPTION_ARG_NONE, &extract_here,
+	  N_("Extract archives using the archive name as destination folder and quit the program"),
+	  NULL },
 
-	{ "default-dir", 0, POPT_ARG_STRING, &default_url, 0,
+	{ "default-dir", '\0', 0, G_OPTION_ARG_STRING, &default_url,
 	  N_("Default folder to use for the '--add' and '--extract' commands"),
 	  N_("FOLDER") },
 
-        { "force", 0, POPT_ARG_NONE, &force_directory_creation, 0,
-          N_("Create destination folder without asking confirmation"),
-          0 },
+	{ "force", '\0', 0, G_OPTION_ARG_NONE, &force_directory_creation,
+	  N_("Create destination folder without asking confirmation"),
+	  NULL },
+
+	{ G_OPTION_REMAINING, 0, 0, G_OPTION_ARG_STRING_ARRAY, &remaining_args, 
+	  NULL, 
+	  NULL },
  
-	{ NULL, '\0', 0, NULL, 0 }
+	{ NULL }
 };
 
 
@@ -167,7 +174,7 @@ struct poptOption options[] = {
 
 
 static guint startup_id = 0;
-static poptContext pctx = NULL; 
+static GOptionContext *context;
 
 
 static gboolean
@@ -177,9 +184,7 @@ startup_cb (gpointer data)
 	startup_id = 0;
 
 	initialize_data ();
-	prepare_app (pctx);
-	poptFreeContext (pctx);
-	pctx = NULL;
+	prepare_app ();
 
 	return FALSE;
 }
@@ -189,15 +194,20 @@ int
 main (int argc, char **argv)
 {
 	GnomeProgram *program;
-	GValue value = { 0 };
+	gchar        *description;
 
 	bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	description = g_strdup_printf ("- %s", _("Create and modify an archive"));
+	context = g_option_context_new (description);
+	g_free (description);
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+
 	program = gnome_program_init ("file-roller", VERSION,
 				      LIBGNOMEUI_MODULE, argc, argv,
-				      GNOME_PARAM_POPT_TABLE, options,
+				      GNOME_PARAM_GOPTION_CONTEXT, context,
 				      GNOME_PARAM_HUMAN_READABLE_NAME, _("Archive Manager"),
 				      GNOME_PARAM_APP_PREFIX, FR_PREFIX,
                                       GNOME_PARAM_APP_SYSCONFDIR, FR_SYSCONFDIR,
@@ -208,11 +218,6 @@ main (int argc, char **argv)
 	g_set_application_name (_("File Roller"));
 	gtk_window_set_default_icon_name ("file-roller");
 
-	g_object_get_property (G_OBJECT (program),
-			       GNOME_PARAM_POPT_CONTEXT,
-			       g_value_init (&value, G_TYPE_POINTER));
-	pctx = g_value_get_pointer (&value);
-
 	if (! gnome_vfs_init ()) 
                 g_error ("Cannot initialize the Virtual File System.");
 
@@ -230,7 +235,7 @@ main (int argc, char **argv)
 
 
 static void 
-initialize_data ()
+initialize_data (void)
 {
 	eel_gconf_monitor_add ("/apps/file-roller");
 	eel_gconf_monitor_add (PREF_NAUTILUS_CLICK_POLICY);
@@ -306,6 +311,7 @@ static void
 release_data ()
 {
 	g_hash_table_destroy (programs_cache);
+	file_data_release_data ();
 
 	eel_global_client_free ();
 
@@ -451,7 +457,7 @@ compute_supported_archive_types (void)
 
 
 static void 
-prepare_app (poptContext pctx)
+prepare_app (void)
 {
 	char *path;
 	char *default_dir;
@@ -481,7 +487,7 @@ prepare_app (poptContext pctx)
 
 	/**/
 
-	if (poptPeekArg (pctx) == NULL) { /* No archive specified. */
+	if (remaining_args == NULL) { /* No archive specified. */
 		FRWindow *window;
 		window = window_new ();
 		gtk_widget_show (window->app);
@@ -490,7 +496,7 @@ prepare_app (poptContext pctx)
 
 	default_dir = get_path_from_url (default_url);
 
-	if ((extract_here == 0) && (extract_to != NULL)) {
+	if (extract_to != NULL) {
 		if (g_path_is_absolute (extract_to))
 			extract_to_path = get_path_from_url (extract_to);
 		else {
@@ -513,12 +519,13 @@ prepare_app (poptContext pctx)
 		FRWindow    *window;
 		GList       *file_list = NULL;
 		const char  *filename;
+		int          i = 0;
 
 		window = window_new ();
 		if (default_dir != NULL)
 			window_set_default_dir (window, default_dir, TRUE);
 		
-		while ((filename = poptGetArg (pctx)) != NULL) {
+		while ((filename = remaining_args[i++]) != NULL) {
 			char *path;
 			
 			if (! g_path_is_absolute (filename)) {
@@ -544,33 +551,30 @@ prepare_app (poptContext pctx)
 		   || (extract_here == 1)) { /* Extract all archives. */
 		FRWindow   *window;
 		const char *archive;
+		int         i = 0;
 		
 		window = window_new ();
 		if (default_dir != NULL)
 			window_set_default_dir (window, default_dir, TRUE);
 
-		while ((archive = poptGetArg (pctx)) != NULL) {
-			if (extract_here == 1) {
-				g_free (extract_to_path);
-				extract_to_path = g_strconcat (archive, "_FILES", NULL);
-			}
-				
-			window_archive__open_extract (window, 
-						      archive, 
-						      extract_to_path);
-
-			if (extract_here == 1) {
-				g_free (extract_to_path);
-				extract_to_path = NULL;
-			}
+		while ((archive = remaining_args[i++]) != NULL) {
+			if (extract_here == 1) 
+				window_archive__open_extract_here (window, 
+								   archive, 
+								   extract_to_path);
+			else
+				window_archive__open_extract (window, 
+							      archive, 
+							      extract_to_path);
 		}
 		window_archive__quit (window);
 		window_batch_mode_start (window);
 
-	} else { /* Open each archives in a window */
+	} else { /* Open each archive in a window */
 		const char *archive;
 		
-		while ((archive = poptGetArg (pctx)) != NULL) {
+		int i = 0;
+		while ((archive = remaining_args[i++]) != NULL) {
 			FRWindow *window;
 			
 			window = window_new ();
